# Form implementation generated from reading ui file 'edit_metadata_summary_dialog_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_EditMetadataSummaryDialog(object):
  def setupUi(self, EditMetadataSummaryDialog):
    EditMetadataSummaryDialog.setObjectName("EditMetadataSummaryDialog")
    EditMetadataSummaryDialog.resize(1047, 486)
    self.gridLayout = QtWidgets.QGridLayout(EditMetadataSummaryDialog)
    self.gridLayout.setObjectName("gridLayout")
    self.buttonBox = QtWidgets.QDialogButtonBox(parent=EditMetadataSummaryDialog)
    self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
    self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.No|QtWidgets.QDialogButtonBox.StandardButton.Yes)
    self.buttonBox.setObjectName("buttonBox")
    self.gridLayout.addWidget(self.buttonBox, 1, 0, 1, 1)
    self.summaryTextEdit = QtWidgets.QTextEdit(parent=EditMetadataSummaryDialog)
    self.summaryTextEdit.setObjectName("summaryTextEdit")
    self.gridLayout.addWidget(self.summaryTextEdit, 0, 0, 1, 1)

    self.retranslateUi(EditMetadataSummaryDialog)
    self.buttonBox.accepted.connect(EditMetadataSummaryDialog.accept) # type: ignore
    self.buttonBox.rejected.connect(EditMetadataSummaryDialog.reject) # type: ignore
    QtCore.QMetaObject.connectSlotsByName(EditMetadataSummaryDialog)

  def retranslateUi(self, EditMetadataSummaryDialog):
    _translate = QtCore.QCoreApplication.translate
    EditMetadataSummaryDialog.setWindowTitle(_translate("EditMetadataSummaryDialog", "Metadata Summary"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EditMetadataSummaryDialog = QtWidgets.QDialog()
    ui = Ui_EditMetadataSummaryDialog()
    ui.setupUi(EditMetadataSummaryDialog)
    EditMetadataSummaryDialog.show()
    sys.exit(app.exec())
