# Form implementation generated from reading ui file 'edit_metadata_dialog_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_EditMetadataDialog(object):
  def setupUi(self, EditMetadataDialog):
    EditMetadataDialog.setObjectName("EditMetadataDialog")
    EditMetadataDialog.resize(950, 428)
    self.verticalLayout = QtWidgets.QVBoxLayout(EditMetadataDialog)
    self.verticalLayout.setObjectName("verticalLayout")
    self.mainVerticalLayout = QtWidgets.QVBoxLayout()
    self.mainVerticalLayout.setObjectName("mainVerticalLayout")
    self.minimalFullComboBox = QtWidgets.QComboBox(parent=EditMetadataDialog)
    self.minimalFullComboBox.setObjectName("minimalFullComboBox")
    self.mainVerticalLayout.addWidget(self.minimalFullComboBox)
    self.horizontalLayout = QtWidgets.QHBoxLayout()
    self.horizontalLayout.setObjectName("horizontalLayout")
    self.label = QtWidgets.QLabel(parent=EditMetadataDialog)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
    self.label.setSizePolicy(sizePolicy)
    self.label.setMinimumSize(QtCore.QSize(100, 0))
    self.label.setObjectName("label")
    self.horizontalLayout.addWidget(self.label)
    self.licenseNameLineEdit = QtWidgets.QLineEdit(parent=EditMetadataDialog)
    self.licenseNameLineEdit.setClearButtonEnabled(True)
    self.licenseNameLineEdit.setObjectName("licenseNameLineEdit")
    self.horizontalLayout.addWidget(self.licenseNameLineEdit)
    self.licenseURLLineEdit = QtWidgets.QLineEdit(parent=EditMetadataDialog)
    self.licenseURLLineEdit.setClearButtonEnabled(True)
    self.licenseURLLineEdit.setObjectName("licenseURLLineEdit")
    self.horizontalLayout.addWidget(self.licenseURLLineEdit)
    self.mainVerticalLayout.addLayout(self.horizontalLayout)
    self.metadataBlockComboBox = QtWidgets.QComboBox(parent=EditMetadataDialog)
    self.metadataBlockComboBox.setObjectName("metadataBlockComboBox")
    self.mainVerticalLayout.addWidget(self.metadataBlockComboBox)
    self.typesComboBox = QtWidgets.QComboBox(parent=EditMetadataDialog)
    self.typesComboBox.setObjectName("typesComboBox")
    self.mainVerticalLayout.addWidget(self.typesComboBox)
    self.metadataScrollArea = QtWidgets.QScrollArea(parent=EditMetadataDialog)
    self.metadataScrollArea.setWidgetResizable(True)
    self.metadataScrollArea.setObjectName("metadataScrollArea")
    self.scrollAreaWidgetContents = QtWidgets.QWidget()
    self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 928, 249))
    self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
    self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
    self.verticalLayout_3.setObjectName("verticalLayout_3")
    self.metadataScrollVerticalLayout = QtWidgets.QVBoxLayout()
    self.metadataScrollVerticalLayout.setObjectName("metadataScrollVerticalLayout")
    self.verticalLayout_3.addLayout(self.metadataScrollVerticalLayout)
    self.metadataScrollArea.setWidget(self.scrollAreaWidgetContents)
    self.mainVerticalLayout.addWidget(self.metadataScrollArea)
    self.verticalLayout.addLayout(self.mainVerticalLayout)
    self.buttonBox = QtWidgets.QDialogButtonBox(parent=EditMetadataDialog)
    self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
    self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Save)
    self.buttonBox.setObjectName("buttonBox")
    self.verticalLayout.addWidget(self.buttonBox)

    self.retranslateUi(EditMetadataDialog)
    self.buttonBox.accepted.connect(EditMetadataDialog.accept) # type: ignore
    self.buttonBox.rejected.connect(EditMetadataDialog.reject) # type: ignore
    QtCore.QMetaObject.connectSlotsByName(EditMetadataDialog)

  def retranslateUi(self, EditMetadataDialog):
    _translate = QtCore.QCoreApplication.translate
    EditMetadataDialog.setWindowTitle(_translate("EditMetadataDialog", "Edit metadata"))
    EditMetadataDialog.setToolTip(_translate("EditMetadataDialog", "<html><head/><body><p><span style=\" font-style:italic;\">Edit the full/minimal set of metadata information for the datasets created in dataverse. Datasets corresponds to PASTA projects and needs to be mapped to the appropriate PASTA level properties as needed.</span></p></body></html>"))
    self.minimalFullComboBox.setToolTip(_translate("EditMetadataDialog", "Select either \"minimal\" or \"full\" from the given list. Below UI will be updated accordingly."))
    self.label.setText(_translate("EditMetadataDialog", "License"))
    self.licenseNameLineEdit.setToolTip(_translate("EditMetadataDialog", "Enter the license name. Default: CC0 1.0"))
    self.licenseNameLineEdit.setPlaceholderText(_translate("EditMetadataDialog", "Enter the license name."))
    self.licenseURLLineEdit.setToolTip(_translate("EditMetadataDialog", "Provide the license URL. Default: http://creativecommons.org/publicdomain/zero/1.0"))
    self.licenseURLLineEdit.setPlaceholderText(_translate("EditMetadataDialog", "Enter the license URL."))
    self.metadataBlockComboBox.setToolTip(_translate("EditMetadataDialog", "Select the metadata block to be edited."))
    self.typesComboBox.setToolTip(_translate("EditMetadataDialog", "Select the metadata block type to be edited."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EditMetadataDialog = QtWidgets.QDialog()
    ui = Ui_EditMetadataDialog()
    ui.setupUi(EditMetadataDialog)
    EditMetadataDialog.show()
    sys.exit(app.exec())
