# Form implementation generated from reading ui file 'upload_widget_base.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide6 import QtCore, QtGui, QtWidgets


class Ui_UploadWidgetFrame(object):
  def setupUi(self, UploadWidgetFrame):
    UploadWidgetFrame.setObjectName("UploadWidgetFrame")
    UploadWidgetFrame.resize(1141, 228)
    UploadWidgetFrame.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
    UploadWidgetFrame.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
    self.gridLayout = QtWidgets.QGridLayout(UploadWidgetFrame)
    self.gridLayout.setObjectName("gridLayout")
    self.uploadWidgetMainverticalLayout = QtWidgets.QVBoxLayout()
    self.uploadWidgetMainverticalLayout.setObjectName("uploadWidgetMainverticalLayout")
    self.uploadWidgetHorizontalLayout = QtWidgets.QHBoxLayout()
    self.uploadWidgetHorizontalLayout.setObjectName("uploadWidgetHorizontalLayout")
    self.uploadProjectLabel = QtWidgets.QLabel(parent=UploadWidgetFrame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.uploadProjectLabel.sizePolicy().hasHeightForWidth())
    self.uploadProjectLabel.setSizePolicy(sizePolicy)
    self.uploadProjectLabel.setMinimumSize(QtCore.QSize(300, 0))
    self.uploadProjectLabel.setMaximumSize(QtCore.QSize(300, 100))
    self.uploadProjectLabel.setObjectName("uploadProjectLabel")
    self.uploadWidgetHorizontalLayout.addWidget(self.uploadProjectLabel)
    self.uploadProgressBar = QtWidgets.QProgressBar(parent=UploadWidgetFrame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.uploadProgressBar.sizePolicy().hasHeightForWidth())
    self.uploadProgressBar.setSizePolicy(sizePolicy)
    self.uploadProgressBar.setMinimumSize(QtCore.QSize(400, 0))
    self.uploadProgressBar.setProperty("value", 0)
    self.uploadProgressBar.setObjectName("uploadProgressBar")
    self.uploadWidgetHorizontalLayout.addWidget(self.uploadProgressBar)
    self.statusIconLabel = QtWidgets.QLabel(parent=UploadWidgetFrame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Fixed, QtWidgets.QSizePolicy.Policy.Preferred)
    sizePolicy.setHorizontalStretch(50)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.statusIconLabel.sizePolicy().hasHeightForWidth())
    self.statusIconLabel.setSizePolicy(sizePolicy)
    self.statusIconLabel.setMinimumSize(QtCore.QSize(25, 25))
    self.statusIconLabel.setText("")
    self.statusIconLabel.setObjectName("statusIconLabel")
    self.uploadWidgetHorizontalLayout.addWidget(self.statusIconLabel)
    self.statusLabel = QtWidgets.QLabel(parent=UploadWidgetFrame)
    self.statusLabel.setMinimumSize(QtCore.QSize(0, 0))
    self.statusLabel.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
    self.statusLabel.setObjectName("statusLabel")
    self.uploadWidgetHorizontalLayout.addWidget(self.statusLabel)
    self.showLogPushButton = QtWidgets.QPushButton(parent=UploadWidgetFrame)
    self.showLogPushButton.setMinimumSize(QtCore.QSize(100, 0))
    icon = QtGui.QIcon.fromTheme("document-page-setup")
    self.showLogPushButton.setIcon(icon)
    self.showLogPushButton.setObjectName("showLogPushButton")
    self.uploadWidgetHorizontalLayout.addWidget(self.showLogPushButton)
    self.uploadCancelPushButton = QtWidgets.QPushButton(parent=UploadWidgetFrame)
    self.uploadCancelPushButton.setMinimumSize(QtCore.QSize(100, 0))
    icon = QtGui.QIcon.fromTheme("window-close")
    self.uploadCancelPushButton.setIcon(icon)
    self.uploadCancelPushButton.setObjectName("uploadCancelPushButton")
    self.uploadWidgetHorizontalLayout.addWidget(self.uploadCancelPushButton)
    self.uploadWidgetMainverticalLayout.addLayout(self.uploadWidgetHorizontalLayout)
    self.logConsoleTextEdit = QtWidgets.QTextEdit(parent=UploadWidgetFrame)
    self.logConsoleTextEdit.setEnabled(True)
    self.logConsoleTextEdit.setMinimumSize(QtCore.QSize(0, 150))
    self.logConsoleTextEdit.setReadOnly(True)
    self.logConsoleTextEdit.setObjectName("logConsoleTextEdit")
    self.uploadWidgetMainverticalLayout.addWidget(self.logConsoleTextEdit)
    self.modelIdLabel = QtWidgets.QLabel(parent=UploadWidgetFrame)
    self.modelIdLabel.setText("")
    self.modelIdLabel.setObjectName("modelIdLabel")
    self.uploadWidgetMainverticalLayout.addWidget(self.modelIdLabel)
    self.gridLayout.addLayout(self.uploadWidgetMainverticalLayout, 0, 0, 1, 1)

    self.retranslateUi(UploadWidgetFrame)
    QtCore.QMetaObject.connectSlotsByName(UploadWidgetFrame)

  def retranslateUi(self, UploadWidgetFrame):
    _translate = QtCore.QCoreApplication.translate
    UploadWidgetFrame.setWindowTitle(_translate("UploadWidgetFrame", "Frame"))
    self.uploadProjectLabel.setToolTip(_translate("UploadWidgetFrame", "Name of the PASTA project enqueued for dataverse upload."))
    self.uploadProjectLabel.setText(_translate("UploadWidgetFrame", "Example Project 1"))
    self.uploadProgressBar.setToolTip(_translate("UploadWidgetFrame", "Shows the upload progress."))
    self.statusLabel.setToolTip(_translate("UploadWidgetFrame", "Displays the status of the upload process (Queued/Success/Failed/In-progress)."))
    self.statusLabel.setText(_translate("UploadWidgetFrame", "Queued..."))
    self.showLogPushButton.setToolTip(_translate("UploadWidgetFrame", "Click to view the log for this particular upload."))
    self.showLogPushButton.setText(_translate("UploadWidgetFrame", "Show Log"))
    self.uploadCancelPushButton.setToolTip(_translate("UploadWidgetFrame", "Click to cancel this particular upload."))
    self.uploadCancelPushButton.setText(_translate("UploadWidgetFrame", "Cancel"))
    self.logConsoleTextEdit.setToolTip(_translate("UploadWidgetFrame", "Displays the generated log messages for this particular upload."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    UploadWidgetFrame = QtWidgets.QFrame()
    ui = Ui_UploadWidgetFrame()
    ui.setupUi(UploadWidgetFrame)
    UploadWidgetFrame.show()
    sys.exit(app.exec())
